@@ -2,7 +2,7 @@ from __future__ import annotations
 
 from dataclasses import dataclass, field
 from enum import Enum as PyEnum
-from typing import Optional, Union
+from typing import List, Union
 
 
 class FieldCardinality(str, PyEnum):
@@ -14,7 +14,7 @@ class FieldCardinality(str, PyEnum):
 # file: BYTE_ORDER_MARK? syntaxDecl? fileElement* EOF;
 @dataclass
 class File:
-    syntax: str | None = None
+    syntax: Union[str, None] = None
     file_elements: list[FileElement] = field(default_factory=list)
 
 
@@ -38,11 +38,23 @@ class Import:
     public: bool = False
 
 
+@dataclass
+class MessageLiteralField:
+    name: str
+    value: MessageValue
+
+
+@dataclass
+class MessageLiteral:
+    fields: List[MessageLiteralField] = field(default_factory=list)
+
+
 # optionDecl: OPTION optionName EQUALS optionValue SEMICOLON;
 @dataclass
+@dataclass
 class Option:
     name: str
-    value: str | int | float | bool | Identifier
+    value: Union[ScalarValue, MessageLiteral]
 
 
 # messageDecl: MESSAGE messageName L_BRACE messageElement* R_BRACE;
@@ -60,7 +72,7 @@ class Field:
     name: str
     number: int
     type: str
-    cardinality: FieldCardinality | None = None
+    cardinality: Union[FieldCardinality, None] = None
     options: list[Option] = field(default_factory=list)
 
 
@@ -81,7 +93,7 @@ class MapField:
 class Group:
     name: str
     number: int
-    cardinality: FieldCardinality | None = None
+    cardinality: Union[FieldCardinality, None] = None
     elements: list[MessageElement] = field(default_factory=list)
 
 
@@ -223,7 +235,7 @@ EnumElement = Union[Option, EnumValue, EnumReserved, Comment]
 
 # extensionElement: extensionFieldDecl |
 #                     groupDecl;
-ExtensionElement = Union[Field, Group]
+ExtensionElement = Union[Field, Group, Comment]
 
 # serviceElement: optionDecl |
 #                   methodDecl |
@@ -235,3 +247,9 @@ ServiceElement = Union[Option, Method, Comment]
 #                 commentDecl |
 #                 emptyDecl;
 MethodElement = Union[Option, Comment]
+
+# Define a type alias for scalar values
+ScalarValue = Union[str, int, float, bool, Identifier]
+
+# Define a recursive type alias for message values
+MessageValue = Union[ScalarValue, MessageLiteral, List["MessageValue"]]
